"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.rad2deg = exports.deg2rad = exports.collisionVelocity = exports.getParticleBaseVelocity = exports.getDistance = exports.getDistances = exports.setRangeValue = exports.getRangeMax = exports.getRangeMin = exports.getRangeValue = exports.randomInRange = exports.mix = exports.clamp = void 0;
const Enums_1 = require("../Enums");
const Vector_1 = require("../Core/Particle/Vector");
function clamp(num, min, max) {
    return Math.min(Math.max(num, min), max);
}
exports.clamp = clamp;
function mix(comp1, comp2, weight1, weight2) {
    return Math.floor((comp1 * weight1 + comp2 * weight2) / (weight1 + weight2));
}
exports.mix = mix;
function randomInRange(r) {
    const max = getRangeMax(r);
    let min = getRangeMin(r);
    if (max === min) {
        min = 0;
    }
    return Math.random() * (max - min) + min;
}
exports.randomInRange = randomInRange;
function getRangeValue(value) {
    return typeof value === "number" ? value : randomInRange(value);
}
exports.getRangeValue = getRangeValue;
function getRangeMin(value) {
    return typeof value === "number" ? value : value.min;
}
exports.getRangeMin = getRangeMin;
function getRangeMax(value) {
    return typeof value === "number" ? value : value.max;
}
exports.getRangeMax = getRangeMax;
function setRangeValue(source, value) {
    if (source === value || (value === undefined && typeof source === "number")) {
        return source;
    }
    const min = getRangeMin(source), max = getRangeMax(source);
    return value !== undefined
        ? {
            min: Math.min(min, value),
            max: Math.max(max, value),
        }
        : setRangeValue(min, max);
}
exports.setRangeValue = setRangeValue;
function getDistances(p1, p2) {
    const dx = p1.x - p2.x;
    const dy = p1.y - p2.y;
    return { dx: dx, dy: dy, distance: Math.sqrt(Math.pow(dx, 2) + Math.pow(dy, 2)) };
}
exports.getDistances = getDistances;
function getDistance(pointA, pointB) {
    return getDistances(pointA, pointB).distance;
}
exports.getDistance = getDistance;
function getParticleBaseVelocity(direction) {
    const baseVelocity = Vector_1.Vector.origin;
    baseVelocity.length = 1;
    switch (direction) {
        case Enums_1.MoveDirection.top:
            baseVelocity.angle = -Math.PI / 2;
            break;
        case Enums_1.MoveDirection.topRight:
            baseVelocity.angle = -Math.PI / 4;
            break;
        case Enums_1.MoveDirection.right:
            baseVelocity.angle = 0;
            break;
        case Enums_1.MoveDirection.bottomRight:
            baseVelocity.angle = Math.PI / 4;
            break;
        case Enums_1.MoveDirection.bottom:
            baseVelocity.angle = Math.PI / 2;
            break;
        case Enums_1.MoveDirection.bottomLeft:
            baseVelocity.angle = (3 * Math.PI) / 4;
            break;
        case Enums_1.MoveDirection.left:
            baseVelocity.angle = Math.PI;
            break;
        case Enums_1.MoveDirection.topLeft:
            baseVelocity.angle = (-3 * Math.PI) / 4;
            break;
        case Enums_1.MoveDirection.none:
        default:
            baseVelocity.angle = Math.random() * Math.PI * 2;
            break;
    }
    return baseVelocity;
}
exports.getParticleBaseVelocity = getParticleBaseVelocity;
function collisionVelocity(v1, v2, m1, m2) {
    return Vector_1.Vector.create((v1.x * (m1 - m2)) / (m1 + m2) + (v2.x * 2 * m2) / (m1 + m2), v1.y);
}
exports.collisionVelocity = collisionVelocity;
function deg2rad(deg) {
    return (deg * Math.PI) / 180;
}
exports.deg2rad = deg2rad;
function rad2deg(rad) {
    return (rad * 180) / Math.PI;
}
exports.rad2deg = rad2deg;
