"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OctTree = void 0;
const Parallelepiped_1 = require("./Parallelepiped");
const Sphere_1 = require("./Sphere");
class OctTree {
    constructor(parallelepiped, capacity) {
        this.parallelepiped = parallelepiped;
        this.capacity = capacity;
        this.points = [];
        this.divided = false;
    }
    subdivide() {
        const x = this.parallelepiped.position.x;
        const y = this.parallelepiped.position.y;
        const z = this.parallelepiped.position.z;
        const w = this.parallelepiped.size.width;
        const h = this.parallelepiped.size.height;
        const d = this.parallelepiped.size.depth;
        const capacity = this.capacity;
        this.northEastFront = new OctTree(new Parallelepiped_1.Parallelepiped(x, y, z, w / 2, h / 2, d / 2), capacity);
        this.northEastBack = new OctTree(new Parallelepiped_1.Parallelepiped(x, y, z + d / 2, w / 2, h / 2, d / 2), capacity);
        this.northWestFront = new OctTree(new Parallelepiped_1.Parallelepiped(x + w / 2, y, z, w / 2, h / 2, d / 2), capacity);
        this.northWestBack = new OctTree(new Parallelepiped_1.Parallelepiped(x + w / 2, y, z + d / 2, w / 2, h / 2, d / 2), capacity);
        this.southEastFront = new OctTree(new Parallelepiped_1.Parallelepiped(x, y + h / 2, z, w / 2, h / 2, d / 2), capacity);
        this.southEastBack = new OctTree(new Parallelepiped_1.Parallelepiped(x, y + h / 2, z + d / 2, w / 2, h / 2, d / 2), capacity);
        this.southWestFront = new OctTree(new Parallelepiped_1.Parallelepiped(x + w / 2, y + h / 2, z, w / 2, h / 2, d / 2), capacity);
        this.southWestBack = new OctTree(new Parallelepiped_1.Parallelepiped(x + w / 2, y + h / 2, z + d / 2, w / 2, h / 2, d / 2), capacity);
        this.divided = true;
    }
    insert(point) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j;
        if (!this.parallelepiped.contains(point.position)) {
            return false;
        }
        if (this.points.length < this.capacity) {
            this.points.push(point);
            return true;
        }
        if (!this.divided) {
            this.subdivide();
        }
        return ((_j = (((_a = this.northEastBack) === null || _a === void 0 ? void 0 : _a.insert(point)) ||
            ((_b = this.northEastFront) === null || _b === void 0 ? void 0 : _b.insert(point)) ||
            ((_c = this.northWestBack) === null || _c === void 0 ? void 0 : _c.insert(point)) ||
            ((_d = this.northWestFront) === null || _d === void 0 ? void 0 : _d.insert(point)) ||
            ((_e = this.southEastBack) === null || _e === void 0 ? void 0 : _e.insert(point)) ||
            ((_f = this.southEastFront) === null || _f === void 0 ? void 0 : _f.insert(point)) ||
            ((_g = this.southWestBack) === null || _g === void 0 ? void 0 : _g.insert(point)) ||
            ((_h = this.southWestFront) === null || _h === void 0 ? void 0 : _h.insert(point)))) !== null && _j !== void 0 ? _j : false);
    }
    queryCircle(position, radius) {
        return this.query(new Sphere_1.Sphere(position.x, position.y, position.z, radius));
    }
    queryParallelepiped(position, size) {
        return this.query(new Parallelepiped_1.Parallelepiped(position.x, position.y, position.z, size.width, size.height, size.depth));
    }
    query(range, found) {
        var _a, _b, _c, _d, _e, _f, _g, _h;
        const res = found !== null && found !== void 0 ? found : [];
        if (!range.intersects(this.parallelepiped)) {
            return [];
        }
        else {
            for (const p of this.points) {
                if (!range.contains(p.position)) {
                    continue;
                }
                res.push(p.particle);
            }
            if (this.divided) {
                (_a = this.northEastBack) === null || _a === void 0 ? void 0 : _a.query(range, res);
                (_b = this.northEastFront) === null || _b === void 0 ? void 0 : _b.query(range, res);
                (_c = this.northWestBack) === null || _c === void 0 ? void 0 : _c.query(range, res);
                (_d = this.northWestFront) === null || _d === void 0 ? void 0 : _d.query(range, res);
                (_e = this.southEastBack) === null || _e === void 0 ? void 0 : _e.query(range, res);
                (_f = this.southEastFront) === null || _f === void 0 ? void 0 : _f.query(range, res);
                (_g = this.southWestBack) === null || _g === void 0 ? void 0 : _g.query(range, res);
                (_h = this.southWestFront) === null || _h === void 0 ? void 0 : _h.query(range, res);
            }
        }
        return res;
    }
}
exports.OctTree = OctTree;
