import { MoveDirection } from "../Enums";
import { Vector } from "../Core/Particle/Vector";
export function clamp(num, min, max) {
    return Math.min(Math.max(num, min), max);
}
export function mix(comp1, comp2, weight1, weight2) {
    return Math.floor((comp1 * weight1 + comp2 * weight2) / (weight1 + weight2));
}
export function randomInRange(r) {
    const max = getRangeMax(r);
    let min = getRangeMin(r);
    if (max === min) {
        min = 0;
    }
    return Math.random() * (max - min) + min;
}
export function getRangeValue(value) {
    return typeof value === "number" ? value : randomInRange(value);
}
export function getRangeMin(value) {
    return typeof value === "number" ? value : value.min;
}
export function getRangeMax(value) {
    return typeof value === "number" ? value : value.max;
}
export function setRangeValue(source, value) {
    if (source === value || (value === undefined && typeof source === "number")) {
        return source;
    }
    const min = getRangeMin(source), max = getRangeMax(source);
    return value !== undefined
        ? {
            min: Math.min(min, value),
            max: Math.max(max, value),
        }
        : setRangeValue(min, max);
}
export function getDistances(p1, p2) {
    const dx = p1.x - p2.x;
    const dy = p1.y - p2.y;
    return { dx: dx, dy: dy, distance: Math.sqrt(Math.pow(dx, 2) + Math.pow(dy, 2)) };
}
export function getDistance(pointA, pointB) {
    return getDistances(pointA, pointB).distance;
}
export function getParticleBaseVelocity(direction) {
    const baseVelocity = Vector.origin;
    baseVelocity.length = 1;
    switch (direction) {
        case MoveDirection.top:
            baseVelocity.angle = -Math.PI / 2;
            break;
        case MoveDirection.topRight:
            baseVelocity.angle = -Math.PI / 4;
            break;
        case MoveDirection.right:
            baseVelocity.angle = 0;
            break;
        case MoveDirection.bottomRight:
            baseVelocity.angle = Math.PI / 4;
            break;
        case MoveDirection.bottom:
            baseVelocity.angle = Math.PI / 2;
            break;
        case MoveDirection.bottomLeft:
            baseVelocity.angle = (3 * Math.PI) / 4;
            break;
        case MoveDirection.left:
            baseVelocity.angle = Math.PI;
            break;
        case MoveDirection.topLeft:
            baseVelocity.angle = (-3 * Math.PI) / 4;
            break;
        case MoveDirection.none:
        default:
            baseVelocity.angle = Math.random() * Math.PI * 2;
            break;
    }
    return baseVelocity;
}
export function collisionVelocity(v1, v2, m1, m2) {
    return Vector.create((v1.x * (m1 - m2)) / (m1 + m2) + (v2.x * 2 * m2) / (m1 + m2), v1.y);
}
export function deg2rad(deg) {
    return (deg * Math.PI) / 180;
}
export function rad2deg(rad) {
    return (rad * 180) / Math.PI;
}
