import { Canvas } from "./Canvas";
import type { IAttract, IBubble, IContainerInteractivity, IMovePathGenerator, IRepulse, IShapeDrawer } from "./Interfaces";
import { Particles } from "./Particles";
import { Retina } from "./Retina";
import type { IOptions } from "../Options/Interfaces/IOptions";
import { FrameManager } from "./FrameManager";
import type { RecursivePartial } from "../Types";
import { Options } from "../Options/Classes/Options";
import type { IContainerPlugin } from "./Interfaces";
import { Particle } from "./Particle";
import { Vector } from "./Particle/Vector";
export declare class Container {
    readonly id: string;
    readonly sourceOptions?: RecursivePartial<IOptions> | undefined;
    started: boolean;
    destroyed: boolean;
    pageHidden: boolean;
    lastFrameTime: number;
    fpsLimit: number;
    interactivity: IContainerInteractivity;
    bubble: IBubble;
    repulse: IRepulse;
    attract: IAttract;
    readonly zLayers = 10000;
    readonly fullOptions: Options;
    options: Options;
    readonly retina: Retina;
    readonly canvas: Canvas;
    readonly particles: Particles;
    readonly drawer: FrameManager;
    readonly drawers: Map<string, IShapeDrawer>;
    readonly plugins: Map<string, IContainerPlugin>;
    readonly pathGenerator: IMovePathGenerator;
    private paused;
    private firstStart;
    private drawAnimationFrame?;
    private readonly eventListeners;
    private readonly intersectionObserver?;
    constructor(id: string, sourceOptions?: RecursivePartial<IOptions> | undefined, ...presets: string[]);
    play(force?: boolean): void;
    pause(): void;
    draw(): void;
    getAnimationStatus(): boolean;
    setPath(pathOrGenerator?: IMovePathGenerator | ((particle: Particle) => Vector), init?: () => void, update?: () => void): void;
    destroy(): void;
    exportImg(callback: BlobCallback): void;
    exportImage(callback: BlobCallback, type?: string, quality?: number): void;
    exportConfiguration(): string;
    refresh(): Promise<void>;
    stop(): void;
    loadTheme(name?: string): Promise<void>;
    start(): Promise<void>;
    private init;
    private intersectionManager;
}
