import type { Container } from "./Container";
import type { IBubbleParticleData, ICoordinates, ICoordinates3d, IDelta, IDistance, IHsl, IParticle, IParticleHslAnimation, IParticleLife, IParticleLoops, IParticleSpin, IParticleValueAnimation, IRgb } from "./Interfaces";
import type { IShapeValues } from "../Options/Interfaces/Particles/Shape/IShapeValues";
import type { IParticles } from "../Options/Interfaces/Particles/IParticles";
import { Particles } from "../Options/Classes/Particles/Particles";
import { MoveDirection, MoveDirectionAlt } from "../Enums";
import type { RecursivePartial } from "../Types";
import type { Stroke } from "../Options/Classes/Particles/Stroke";
import { Vector } from "./Particle/Vector";
import { Vector3d } from "./Particle/Vector3d";
export declare class Particle implements IParticle {
    readonly id: number;
    readonly container: Container;
    readonly group?: string | undefined;
    attractDistance?: number;
    close: boolean;
    destroyed: boolean;
    fill: boolean;
    linksDistance?: number;
    linksWidth?: number;
    moveSpeed: number;
    misplaced: boolean;
    orbitRadius?: number;
    orbitRotation?: number;
    randomIndexData?: number;
    sizeAnimationSpeed?: number;
    spawning: boolean;
    splitCount: number;
    unbreakable: boolean;
    zIndexFactor: number;
    readonly pathDelay: number;
    readonly sides: number;
    readonly strokeWidth: number;
    readonly options: Particles;
    readonly direction: MoveDirection | keyof typeof MoveDirection | MoveDirectionAlt;
    readonly life: IParticleLife;
    readonly loops: IParticleLoops;
    readonly spin?: IParticleSpin;
    readonly stroke: Stroke;
    readonly position: Vector3d;
    readonly offset: Vector;
    readonly shadowColor: IRgb | undefined;
    readonly color?: IParticleHslAnimation;
    readonly maxDistance: Partial<IDistance>;
    readonly opacity: IParticleValueAnimation<number>;
    readonly rotate: IParticleValueAnimation<number>;
    readonly size: IParticleValueAnimation<number>;
    readonly strokeColor?: IParticleHslAnimation;
    readonly orbitColor?: IHsl;
    readonly velocity: Vector;
    readonly shape: string;
    readonly initialPosition: Vector3d;
    readonly initialVelocity: Vector;
    readonly shapeData?: IShapeValues;
    readonly bubble: IBubbleParticleData;
    constructor(id: number, container: Container, position?: ICoordinates, overrideOptions?: RecursivePartial<IParticles>, group?: string | undefined);
    draw(delta: IDelta): void;
    getPosition(): ICoordinates3d;
    getRadius(): number;
    getFillColor(): IHsl | undefined;
    getStrokeColor(): IHsl | undefined;
    destroy(override?: boolean): void;
    reset(): void;
    private setColorAnimation;
    private calcPosition;
    fixPositionInCanvas(pos: ICoordinates3d): void;
    private getOutCanvasFix;
    private checkOverlap;
    private calculateVelocity;
    private split;
    private loadShapeData;
}
